{{
********************************
* properly testing with PropIt *
* PropIt Demo                  *
* (C) 2008 Thomas Kliche       *
********************************

Revision History
----------------
 1.1 on 24 January 2008
     - see related changes of file PropIt_110.spin
     - added new test method testPerf to demonstrate performance measurement
     
 1.01 on 16 January 2008
     - see related changes of file PropIt_101.spin

 1.0 on 12 January 2008
     - first public version

...................................................................................................

This is a demonstration for automated testing of a Spin object using PropIt.
For details about PropIt please have a look at the comment in file
PropIt_XXX.spin

Now it provides measurement of the execution time for each test case. So you
can use PropIt to compare the runtime performance of different implementations
of your code. Please have a look at method testPerf if you are interested in
this.  


How can I run this demo?

Please compile this file and save the binary file (F8 then button "Show Hex"
if it's hidden and button "Save Binary File") or use the delivered binary
which is included in this release. Then upload it with PropTerminal to your
Propeller board. If it runs it should display all test cases successful.

Now you can make changes in the file ToTest_XXX.spin. After that you have to
recompile and upload this file again to your Propeller board. If you was
changing the functionality of one of the methods PropIt should display the
related test case as wrong. 

Have fun.

Thomas
alias Kaio


P.S. If you have any question or suggestion please use the Propeller forum.
---------------------------------------------------------------------------------------------------
}}
CON

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

  TCS_PER_METHOD = 100          'specifies the maximum number of test cases per test method
                                'I recommend to use only values times of 10 e.g. 10, 100, 1000,,,
  ERR_TEST_FAILED = -999_999    'error code used by method fail

  SCR_WIDTH      = 40           'specifies the screen width in characters
  SCR_NEWLINE    = $0D          'go to the next begin of line on screen


OBJ

  screen : "PC_Text"            'driver of PropTerminal
  testIt : "ToTest_100"         'your object you want to test


VAR

  long  testCase, testCaseStart, isTestOk, isShowResult, CNV2MICRO


PUB main

  isShowResult := FALSE         'if TRUE the result of each test case
                                'and the execution time will be displayed 
                                'this can be helpfully for further problem finding
  init
  screen.str(@PropItFor)
  screen.str(@_HeadLine1)
  screen.str(@_HeadLine2)
  runTest


PRI init

  CNV2MICRO := clkfreq / 1_000_000 'calculate constant to convert to s
  
  testit.stop                   'reset your object if necessary
  testit.start                  'prepare your object for testing
  screen.stop
  screen.start(12)


PRI runTest | err, nxtTestCase

  repeat SCR_WIDTH - 1
    screen.out("-")
  screen.out(SCR_NEWLINE)

  repeat nxtTestCase from TCS_PER_METHOD {
}                      to 9 * TCS_PER_METHOD {
}                    step TCS_PER_METHOD
    testCase := nxtTestCase
    isTestOk := TRUE
    err := 0

    screen.dec(nxtTestCase)
    screen.out($20)

    case nxtTestCase
      100:                      'you have to change the branch values if you change TCS_PER_METHOD
        err := \test100         'test method which could contain the test cases 100 - 199
                                'please take care to use the backslash in front of your method name
                                'to allow catching of errors via abort instruction by your test object

      200:
        err := \testAdd         'test cases for addition

      300:
        err := \testSub         'test cases for subtraction

      400:
        err := \testPerf        'test cases for performance measurement

      500:

      600:

      700:

      800:

      900:

    if err <> 0
      screen.str(@_Error)
      screen.dec(err)
      screen.str(@_OccuredInTest)
      screen.dec(testCase)
      screen.out(SCR_NEWLINE)
      
      if err == ERR_TEST_FAILED
        abort
    elseif isTestOk
      screen.str(@_Successful)
      screen.out(SCR_NEWLINE)


PRI test100 | testParameter                             'template for a test method
{
  testParameter := 123                                  'prepare parameter for next test case
  firstTestCase                                         'preparation of time measurement for first test case
  result := testIt.myFunction(testParameter)            'test a function with a parameter
  assertTrue(result > expectedValue)                    'check your test result by a condition

  testParameter := 456                                  'prepare parameter for next test case  
  nextTestCase                                          'next test case and preparation for time measurement
  result := testIt.myFunction(testParameter)            'test a function with another parameter
  assertTrue(result == expectedValue)                   'check your test result by a condition

  if (result != expectedValue)                          'if your requirements are not fulfilled
    failMsg(string("test not possible"))                'you can abort the whole test

  nextTestCase                                          'next test case and preparation for time measurement
  result := testIt.otherFunction(testParameter)         'test another related function with a parameter
  assertEquals(expectedValue, result)                   'check your test result if it is equal as expected

  nextTestCase                                          'next test case and preparation for time measurement
  result := testIt.myFunction
  showPerfMessage(string("myFunction"))                 'show result of performance measurement

  result := 0                                           'all ok, to avoid an error message in test loop
}

PRI testAdd                                             'test cases for addition

  firstTestCase                                         'preparation of time measurement for first test case
  result := testIt.addition(71, 29)                     'test case with positive values
  assertEquals(100, result)                             'check if result is equal 100

  nextTestCase
  result := testIt.addition(-23, 47)                    'test case with a negative value as first
  assertTrue(result == 24)                              'you can also use a condition like this

  nextTestCase
  result := testIt.addition(-23, 47)                    'test case with a negative value as second
  assertTrue(result == 24)                              'you can also use a condition like this

  nextTestCase
  result := testIt.addition(-23, -24)                   'test case with negative values
  assertTrueMsg(string("add of two negatives fails"), result == -47)  'you can also provide a custom message
                                                        'which will be displayed in case of an error
  result := 0                                           'all ok, to avoid an error message in test loop


PRI testSub                                             'test cases for subtraction

  firstTestCase                                         'preparation of time measurement for first test case
  result := testIt.subtraction(98, 24)                  'test case with positive values
  assertEqualsMsg(string("sub of two positives fails"), 74, result)  'you can also provide a custom message
                                                        'which will be displayed in case of an error
  nextTestCase
  result := testIt.subtraction(24, 27)                  'test case rolling under 0
  assertEquals(-3, result)

  nextTestCase
  result := testIt.subtraction(64, -24)                 'test case with a negative value as second
  assertTrue(result == 88)

  nextTestCase
  result := testIt.subtraction(-23, -24)                'test case with negative values
  assertTrue(result == 1)                               'check by condition
  
  result := 0                                           'all ok, to avoid an error message in test loop


PRI testPerf | a, b, oldShowResult                      'test cases for performance measurement
'to show execution time of each test case using assertXXX
'you must set isShowResult := TRUE (see method main)  
  oldShowResult := isShowResult                         'save setting
'  isShowResult := TRUE                                  'show result of performance measurement
  
  a := 1357                                             'preparation for local test cases
  b := 123
  
  firstTestCase                                         'preparation of time measurement for first test case
  result := a + b                                       'local test
  assertTrue(result == 1480)                            'performance measurement will be performed by all
                                                        'assertXXX methods, but be aware that the
                                                        'execution time for the condition as parameter 
                                                        'will be included in the measurement result
                                                        'in this case 5 s for the condition
                                                           
' if you want a precisely measurement result use one of the following possibilities
  nextTestCase
  result := a + b                                       'local test
  assertTrue(TRUE)                                      'here no condition is used, but you must set
                                                        'isShowResult := TRUE to show the measurement result
                                                        'see at the begin of this method and the next statement
                                                        'for a way to do this temporarily 
                                                                                     
  isShowResult := oldShowResult                         'restore setting

  nextTestCase
  result := a + b
  showPerfMessage(string("result := a + b"))            'you can also take a measurement and display a message 

  nextTestCase
  showPerfMessage(string("empty test case"))            'this demonstrates that the performance measurement will really
                                                        'only show the time consumption between this two statements
                                                         
  result := 0                                           'all ok, to avoid an error message in test loop


PUB assertEquals(referenceValue, testValue)
{{
Stops the performance measurement and shows a generic
error message if the specified values are not equal.
}}

  if isShowResult
    showResult(testValue, FALSE)

  if referenceValue == testValue
    return
  else
    showFailure(@_TestValueIs, @_NotEqual, referenceValue)


PUB assertEqualsMsg(message, referenceValue, testValue)
{{
Stops the performance measurement and shows the given
message if the specified values are not equal.
}}

  if isShowResult
    showResult(testValue, TRUE)

  if referenceValue == testValue
    return
  else
    showErrMessage(message)


PUB assertNotEquals(referenceValue, testValue)
{{
Stops the performance measurement and shows a generic
error message if the specified values are equal.
}}

  if isShowResult
    showResult(testValue, FALSE)

  if referenceValue <> testValue
    return
  else
    showFailure(@_TestValueIs, @_Equal, referenceValue)


PUB assertNotEqualsMsg(message, referenceValue, testValue)
{{
Stops the performance measurement and shows the given
message if the specified values are equal.
}}

  if isShowResult
    showResult(testValue, TRUE)

  if referenceValue <> testValue
    return
  else
    showErrMessage(message)


PUB assertFalse(condition)
{{
Stops the performance measurement and shows a generic
error message if the specified condition is not false.
}}

  if isShowResult
    showResultBoolean(condition, FALSE)

  if not condition
    return
  else
    showFailureBoolean(@_Condition, @_BooleanFalse)


PUB assertFalseMsg(message, condition)
{{
Stops the performance measurement and shows the given
message if the specified condition is not false.
}}

  if isShowResult
    showResultBoolean(condition, TRUE)

  if not condition
    return
  else
    showErrMessage(message)


PUB assertTrue(condition)
{{
Stops the performance measurement and shows a generic
error message if the specified condition is not true.
}}

  if isShowResult
    showResultBoolean(condition, FALSE)

  if condition
    return
  else
    showFailureBoolean(@_Condition, @_BooleanTrue)


PUB assertTrueMsg(message, condition)
{{
Stops the performance measurement and shows the given
message if the specified condition is not true.
}}

  if isShowResult
    showResultBoolean(condition, TRUE)

  if condition
    return
  else
    showErrMessage(message)


PUB fail
{{
Aborts testing and shows a generic error message.
}}

  failMsg(@_TestFailed)


PUB failMsg(message)
{{
Shows the given message and aborts testing.
}}

  showErrMessage(message)
  abort ERR_TEST_FAILED


PUB firstTestCase
{{
Prepares the first test case for performance measurement.
}}

  testCaseStart += 0            'dummy statement for balanced execution time as nextTestCase      
  testCaseStart := cnt


PUB nextTestCase
{{
Counts the test case and prepares for performance measurement.
}}

  testCase += 1
  testCaseStart := cnt


PUB showMessage(message)
{{
Shows the given message.
Note: This method will not touch the performance measurement,
      but the runtime of this method will be included in the
      measurement result if you would use this method during
      a measurement.
}}

  screen.str(message)
  screen.str(@_InTest)
  screen.dec(testCase)
  screen.out(SCR_NEWLINE)


PUB showPerfMessage(message) | time, testCaseEnd
{{
Stops the performance measurement and shows the given message
and measurement result.
}}

  testCaseEnd := cnt
  time := testCaseEnd - testCaseStart - 1248

  screen.str(message)
  screen.str(@_InTest)
  screen.dec(testCase)
  screen.str(@_At)
  screen.dec(time / CNV2MICRO)
  screen.str(@_Us)
  screen.out(SCR_NEWLINE)


PRI showFailure(msg1, msg2, referenceValue)

  isTestOk := FALSE

  screen.str(msg1)
  screen.str(msg2)
  screen.out($20)
  screen.dec(referenceValue)
  screen.str(@_InTest)
  screen.dec(testCase)
  screen.out(SCR_NEWLINE)


PRI showFailureBoolean(msg1, msg2)

  isTestOk := FALSE

  screen.str(msg1)
  screen.str(@_Not)
  screen.out($20)
  screen.str(msg2)
  screen.str(@_InTest)
  screen.dec(testCase)
  screen.out(SCR_NEWLINE)


PRI showErrMessage(message)

  isTestOk := FALSE
  showMessage(message)


PRI showResult(value, isMsg) | time, testCaseEnd

  testCaseEnd := cnt
  time := testCaseEnd - testCaseStart - 2240
  if isMsg
    time -= 288

  screen.str(@_ValueEqual)
  screen.dec(value)
  screen.str(@_InTest)
  screen.dec(testCase)
  screen.str(@_At)
  screen.dec(time / CNV2MICRO)
  screen.str(@_Us)
  screen.out(SCR_NEWLINE)


PRI showResultBoolean(value, isMsg) | time, testCaseEnd

  testCaseEnd := cnt
  time := testCaseEnd - testCaseStart - 2112
  if isMsg
    time -= 288

  if value == TRUE or value == FALSE
    screen.str(@_ValueEqual)
    screen.str(lookupz (-value: @_BooleanFalse, @_BooleanTrue))
  else
    screen.str(@_WrongBoolean)
    screen.dec(value)
  screen.str(@_InTest)
  screen.dec(testCase)
  screen.str(@_At)
  screen.dec(time / CNV2MICRO)
  screen.str(@_Us)
  screen.out(SCR_NEWLINE)


DAT

PropItFor               byte    "PropIt Demo -"         'enter the name for your test suite
                        byte    " PropIt V1.1",SCR_NEWLINE,SCR_NEWLINE,0
_HeadLine1              byte    "Test  Status",SCR_NEWLINE,0
_HeadLine2              byte    "Case",SCR_NEWLINE,0
_Condition              byte    "condition is ",0
_TestValueIs            byte    "testValue is ",0
_InTest                 byte    " in test ",0
_At                     byte    " at ",0
_Us                     byte    " s",0
_OccuredInTest          byte    " occured during test ",0
_WrongBoolean           byte    "wrong boolean value ",0
_ValueEqual             byte    "Value=",0
_BooleanFalse           byte    "FALSE",0
_BooleanTrue            byte    "TRUE",0
_NotEqual               byte    "not "
_Equal                  byte    "equal",0
_Not                    byte    "not",0
_Error                  byte    "Error ",0
_Successful             byte    "successful",0
_TestFailed             byte    "test failed",0
'