{{
**************************
* properly testing with  *
* PropIt                 *
* (C) 2008 Thomas Kliche *
**************************

Revision History
----------------
 1.1 on 24 January 2008
     - added measurement of execution time for each test case
     - added comments to all public methods 
     - extended code of template method test100 

 1.01 on 16 January 2008
     - fixed negation in assertFalse and assertFalseMsg
     - added check of valid boolean value in showResultBoolean

 1.0 on 12 January 2008
     - first public version

...................................................................................................

This utility provides functionalities for automated testing of a Spin object.
The available methods are like those you can find in JUnit which you can use
to test Java modules.

Now it provides measurement of the execution time for each test case. So you
can use PropIt to compare the runtime performance of different implementations
of your code. Please have a look at method testPerf in PropItDemo if you are
interested in this.  

For one who has never used such test tools, here is a short description as it
works. All others can go over to the next paragraph.


How does automated testing work?

Someone who is reading this was thinking: This tool could help me to find the
errors in my code. You are right, but it only identifies in which function
a problem occur and not where it is located in the code.
Automated testing uses the black box method. The Spin object you want to test
is the black box. You must not know as it works internally. But you know: If
I will get a special result from it I have to give some certain input into it.
Here it is where this tool can support you and you will save much time if you
has made some changes in your code and you will now see if all is working fine.
Each functionality of the black box has to be tested in one or more test case.
Each test case fills the box with a set of input parameters and checks if it
will return the expected result. If the result is wrong a message is displayed
in which test case the problem was occured. This goes on til all test cases
are processed or til a test case fails which is mandatory to proceed.


What I must do?

I assume you have developed a Spin object and your coding is well tested and
all works as you expect. Add your Spin object in the OBJ section of this file
as "testIt". Now you can start to create some test cases to test the
functionality of your code. I suggest to group several test cases which are
logically related together in the same test method, e.g. test methods for
create, change and delete if you would have a file application. In each test
method you can have 100 test cases by default but you can specify more if you
want, see TCS_PER_METHOD. Use the template test method "test100" to create
your own. Write your test cases in your test method like contained in the
template. Add your test method to the related case branch in method "runTest".
Compile this object and upload it with PropTerminal to your Propeller board.
Then you will see the result of your test cases.


How can I measure the execution time of an test case?

PropIt performs a measurement between the method firstTestCase and accordingly
nextTestCase and assertXXX method. You can see the measurement result if you
set isShowResult := TRUE, see at begin of method main. This can be used with
your test cases without change or preparation of your code. A disadvantage in
this case is that the condition which is used as parameter of an assertTrueXXX
or assertFalseXXX method consumes also runtime which is included in the
measurement result.
If you want a precisely measurement result it's better to use the method
showPerfMessage. In this case you can't use it for functional testing.

Have fun.

Thomas
alias Kaio


P.S. If you have any question or suggestion please use the Propeller forum.
     If you think that this utility was a great help for you, your Paypal
     donation via pod-kaio@arcor.de is welcome.
---------------------------------------------------------------------------------------------------
}}
CON

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

  TCS_PER_METHOD  = 100         'specifies the maximum number of test cases per test method
                                'I recommend to use only values times of 10 e.g. 10, 100, 1000,,,
  ERR_TEST_FAILED = -999_999    'error code used by method fail

  SCR_WIDTH       = 40          'specifies the screen width in characters
  SCR_NEWLINE     = $0D         'go to the next begin of line on screen


OBJ

  screen : "PC_Text"            'driver of PropTerminal
  testIt : "ToTest_100"         'your object you want to test


VAR

  long  testCase, testCaseStart, isTestOk, isShowResult, CNV2MICRO


PUB main

  isShowResult := FALSE         'if TRUE the result of each test case
                                'and the execution time will be displayed 
                                'this can be helpfully for further problem finding
  init
  screen.str(@PropItFor)
  screen.str(@_HeadLine1)
  screen.str(@_HeadLine2)
  runTest


PRI init
  
  CNV2MICRO := clkfreq / 1_000_000 'calculate constant to convert to s
  
  testIt.stop                   'reset your object if necessary
  testIt.start                  'prepare your object for testing
  screen.stop
  screen.start(12)


PRI runTest | err, nxtTestCase

  repeat SCR_WIDTH - 1
    screen.out("-")
  screen.out(SCR_NEWLINE)

  repeat nxtTestCase from TCS_PER_METHOD {
}                      to 9 * TCS_PER_METHOD {
}                    step TCS_PER_METHOD
    testCase := nxtTestCase
    isTestOk := TRUE
    err := 0

    screen.dec(nxtTestCase)
    screen.out($20)

    case nxtTestCase
      100:                      'you have to change the branch values if you change TCS_PER_METHOD
        err := \test100         'test method which could contain the test cases 100 - 199
                                'please take care to use the backslash in front of your method name
                                'to allow catching of errors via abort instruction by your test object

      200:

      300:

      400:

      500:

      600:

      700:

      800:

      900:

    if err <> 0
      screen.str(@_Error)
      screen.dec(err)
      screen.str(@_OccuredInTest)
      screen.dec(testCase)
      screen.out(SCR_NEWLINE)

      if err == ERR_TEST_FAILED
        abort
    elseif isTestOk
      screen.str(@_Successful)
      screen.out(SCR_NEWLINE)


PRI test100 | testParameter                             'template for a test method
{
  testParameter := 123                                  'prepare parameter for next test case
  firstTestCase                                         'preparation of time measurement for first test case
  result := testIt.myFunction(testParameter)            'test a function with a parameter
  assertTrue(result > expectedValue)                    'check your test result by a condition

  testParameter := 456                                  'prepare parameter for next test case  
  nextTestCase                                          'next test case and preparation for time measurement
  result := testIt.myFunction(testParameter)            'test a function with another parameter
  assertTrue(result == expectedValue)                   'check your test result by a condition

  if (result != expectedValue)                          'if your requirements are not fulfilled
    failMsg(string("test not possible"))                'you can abort the whole test

  nextTestCase                                          'next test case and preparation for time measurement
  result := testIt.otherFunction(testParameter)         'test another related function with a parameter
  assertEquals(expectedValue, result)                   'check your test result if it is equal as expected

  nextTestCase                                          'next test case and preparation for time measurement
  result := testIt.myFunction
  showPerfMessage(string("myFunction"))                 'show result of performance measurement

  result := 0                                           'all ok, to avoid an error message in test loop
}

PUB assertEquals(referenceValue, testValue)
{{
Stops the performance measurement and shows a generic
error message if the specified values are not equal.
}}

  if isShowResult
    showResult(testValue, FALSE)

  if referenceValue == testValue
    return
  else
    showFailure(@_TestValueIs, @_NotEqual, referenceValue)


PUB assertEqualsMsg(message, referenceValue, testValue)
{{
Stops the performance measurement and shows the given
message if the specified values are not equal.
}}

  if isShowResult
    showResult(testValue, TRUE)

  if referenceValue == testValue
    return
  else
    showErrMessage(message)


PUB assertNotEquals(referenceValue, testValue)
{{
Stops the performance measurement and shows a generic
error message if the specified values are equal.
}}

  if isShowResult
    showResult(testValue, FALSE)

  if referenceValue <> testValue
    return
  else
    showFailure(@_TestValueIs, @_Equal, referenceValue)


PUB assertNotEqualsMsg(message, referenceValue, testValue)
{{
Stops the performance measurement and shows the given
message if the specified values are equal.
}}

  if isShowResult
    showResult(testValue, TRUE)

  if referenceValue <> testValue
    return
  else
    showErrMessage(message)


PUB assertFalse(condition)
{{
Stops the performance measurement and shows a generic
error message if the specified condition is not false.
}}

  if isShowResult
    showResultBoolean(condition, FALSE)

  if not condition
    return
  else
    showFailureBoolean(@_Condition, @_BooleanFalse)


PUB assertFalseMsg(message, condition)
{{
Stops the performance measurement and shows the given
message if the specified condition is not false.
}}

  if isShowResult
    showResultBoolean(condition, TRUE)

  if not condition
    return
  else
    showErrMessage(message)


PUB assertTrue(condition)
{{
Stops the performance measurement and shows a generic
error message if the specified condition is not true.
}}

  if isShowResult
    showResultBoolean(condition, FALSE)

  if condition
    return
  else
    showFailureBoolean(@_Condition, @_BooleanTrue)


PUB assertTrueMsg(message, condition)
{{
Stops the performance measurement and shows the given
message if the specified condition is not true.
}}

  if isShowResult
    showResultBoolean(condition, TRUE)

  if condition
    return
  else
    showErrMessage(message)


PUB fail
{{
Aborts testing and shows a generic error message.
}}

  failMsg(@_TestFailed)


PUB failMsg(message)
{{
Shows the given message and aborts testing.
}}

  showErrMessage(message)
  abort ERR_TEST_FAILED


PUB firstTestCase
{{
Prepares the first test case for performance measurement.
}}

  testCaseStart += 0            'dummy statement for balanced execution time as nextTestCase      
  testCaseStart := cnt


PUB nextTestCase
{{
Counts the test case and prepares for performance measurement.
}}

  testCase += 1
  testCaseStart := cnt


PUB showMessage(message)
{{
Shows the given message.
Note: This method will not touch the performance measurement,
      but the runtime of this method will be included in the
      measurement result if you would use this method during
      a measurement.
}}

  screen.str(message)
  screen.str(@_InTest)
  screen.dec(testCase)
  screen.out(SCR_NEWLINE)


PUB showPerfMessage(message) | time, testCaseEnd
{{
Stops the performance measurement and shows the given message
and measurement result.
}}

  testCaseEnd := cnt
  time := testCaseEnd - testCaseStart - 1248

  screen.str(message)
  screen.str(@_InTest)
  screen.dec(testCase)
  screen.str(@_At)
  screen.dec(time / CNV2MICRO)
  screen.str(@_Us)
  screen.out(SCR_NEWLINE)


PRI showFailure(msg1, msg2, referenceValue)

  isTestOk := FALSE

  screen.str(msg1)
  screen.str(msg2)
  screen.out($20)
  screen.dec(referenceValue)
  screen.str(@_InTest)
  screen.dec(testCase)
  screen.out(SCR_NEWLINE)


PRI showFailureBoolean(msg1, msg2)

  isTestOk := FALSE

  screen.str(msg1)
  screen.str(@_Not)
  screen.out($20)
  screen.str(msg2)
  screen.str(@_InTest)
  screen.dec(testCase)
  screen.out(SCR_NEWLINE)


PRI showErrMessage(message)

  isTestOk := FALSE
  showMessage(message)


PRI showResult(value, isMsg) | time, testCaseEnd

  testCaseEnd := cnt
  time := testCaseEnd - testCaseStart - 2240
  if isMsg
    time -= 288

  screen.str(@_ValueEqual)
  screen.dec(value)
  screen.str(@_InTest)
  screen.dec(testCase)
  screen.str(@_At)
  screen.dec(time / CNV2MICRO)
  screen.str(@_Us)
  screen.out(SCR_NEWLINE)


PRI showResultBoolean(value, isMsg) | time, testCaseEnd

  testCaseEnd := cnt
  time := testCaseEnd - testCaseStart - 2112
  if isMsg
    time -= 288

  if value == TRUE or value == FALSE
    screen.str(@_ValueEqual)
    screen.str(lookupz (-value: @_BooleanFalse, @_BooleanTrue))
  else
    screen.str(@_WrongBoolean)
    screen.dec(value)
  screen.str(@_InTest)
  screen.dec(testCase)
  screen.str(@_At)
  screen.dec(time / CNV2MICRO)
  screen.str(@_Us)
  screen.out(SCR_NEWLINE)


DAT

PropItFor               byte    "properly testing with"   'enter the name for your test suite
                        byte    " PropIt V1.1",SCR_NEWLINE,SCR_NEWLINE,0
_HeadLine1              byte    "Test  Status",SCR_NEWLINE,0
_HeadLine2              byte    "Case",SCR_NEWLINE,0
_Condition              byte    "condition is ",0
_TestValueIs            byte    "testValue is ",0
_InTest                 byte    " in test ",0
_At                     byte    " at ",0
_Us                     byte    " s",0
_OccuredInTest          byte    " occured during test ",0
_WrongBoolean           byte    "wrong boolean value ",0
_ValueEqual             byte    "Value=",0
_BooleanFalse           byte    "FALSE",0
_BooleanTrue            byte    "TRUE",0
_NotEqual               byte    "not "
_Equal                  byte    "equal",0
_Not                    byte    "not",0
_Error                  byte    "Error ",0
_Successful             byte    "successful",0
_TestFailed             byte    "test failed",0
'